#!/usr/bin/python
# -*- coding: utf-8 -*-

import time
import sys
from os import path
from httplib2 import Http
from urllib import urlencode
from math import log,tan,pi,cos,radians,ceil,floor
from lxml import etree
from lxml.builder import ElementMaker
from PIL import Image, ImageDraw
from StringIO import StringIO
from geohash import encode
from geographiclib.geodesic import Geodesic
from treecutter import constants as const

# EC Equator lenght
EC = 40075016.686 # meter
# ER Earth radius
ER = 6372798.2 # meter
# Availible zoom levels in OSM
ZOOMRANGE = range(1, 18)
# tile size
TS = 256

h = Http(".cache")

class Coord(object):
    def __init__(self, lat, lon):
        self.latitude  = float(lat)
        self.longitude = float(lon)
        self.image = None

    def osmlink(self):
        return "http://www.openstreetmap.org/?mlat=%s&mlon=%s&zoom=18&layers=M"\
            % (self.latitude,self.longitude)

    def dms(self):
        ns = self.latitude
        ew = self.longitude
        mnt,sec = divmod(ns*3600,60)
        deg,mnt = divmod(mnt,60)
        out = u'''%d°%2d'%5.2f"%s''' % ( deg,mnt,sec,'N')
        mnt,sec = divmod(ew*3600,60)
        deg,mnt = divmod(mnt,60)
        out +=  u''' %d°%2d'%05.2f"%s''' % ( deg,mnt,sec,'E')
        return out

    def lontile(self, zoom):
        tile = ((self.longitude + 180) / 360) * (2**zoom)
        return tile

    def lattile(self, zoom):
        rad = radians(self.latitude)
        tile = (1-log(tan(rad)+1/cos(rad))/pi)/2*2**zoom
        return tile

    def offset(self,zoom):
        x = self.lontile(zoom)
        y = self.lattile(zoom)
        xo = int(floor((x-floor(x))*TS))
        yo = int(floor((y-floor(y))*TS))
        return (xo, yo)

    def distance(self, point):
        res = Geodesic.WGS84.Inverse(self.latitude, self.longitude,
                                     point.latitude, point.longitude)
        return res['s12']

    def direct(self, direction, lenght):
        point = Geodesic.WGS84.Direct(self.latitude, self.longitude,
                                      direction, length)
        return self.__class__(point['lat2'],point['lon2'])

    def png(self,zoom=15,size=(400,150)):
        filename = encode(self.latitude, self.longitude)+'.png'
#        if path.isfile(filename):
#            if path.getctime(filename) > time.time() - 60*60*24*2:
#                return
        im = Image.new("RGB", size, None)

        ew = int(self.lontile(zoom))
        ns = int(self.lattile(zoom))

        (xo, yo) = self.offset(zoom)
        et = int(floor((xo - ceil(size[0]/2))/TS))
        nt = int(floor((yo - ceil(size[1]/2))/TS))
        wt = int(floor((xo + ceil(size[0]/2))/TS))
        st = int(floor((yo + ceil(size[1]/2))/TS))

        lontiles =  range(ew+et,ew+wt+1)
        lattiles = range(ns+nt,ns+st+1)
        imsize = (len(lontiles)*TS,len(lattiles)*TS)
        grid = Image.new("RGB", imsize, None)
        for yi, y in enumerate(lattiles):
            for xi, x in enumerate(lontiles):
                url = 'http://tile.openstreetmap.org/%s/%s/%s.png' % (zoom,x,y)
                request, content = h.request(url)
                img = Image.open(StringIO(content))
#                dr = ImageDraw.Draw(img)
#                dr.rectangle([0,0,TS,TS], outline=0)
                box = (xi*TS, yi*TS)
                grid.paste(img, box)

        yp = [i for i,j in enumerate(lattiles) if j == int(ns)][0]*TS+yo
        xp = [i for i,j in enumerate(lontiles) if j == int(ew)][0]*TS+xo
        mark(grid, (xp,yp))
        xc = int(ceil(size[0]/2))
        yc = int(ceil(size[1]/2))

#        draw = ImageDraw.Draw(grid)
#        draw.rectangle([xp-xc,yp-yc,xp+xc,yp+yc], outline="red")
        gridc = grid.crop((xp-xc,yp-yc,xp+xc,yp+yc))
        gridc.save(filename)

    def db_xml(self):
        self.png()
        img = encode(self.latitude, self.longitude)+'.png'
        phr = "geo:"+str(self.latitude)+","+str(self.longitude)

        db = ElementMaker(namespace=const.DB_NS, nsmap=const.NSMAP)
        uri = db.uri(db.link(
                db.inlinemediaobject(
                    db.imageobject(db.imagedata(
                            fileref=img,
                            format='PNG')),
                    db.textobject(db.phrase(phr))
                    ),
                self.dms(),
                **{const.XLINK+"href": self.osmlink()}))
        return uri

def c(s):
    return s or ''

def s(s,n):
    if n is not None:
        return s or n.text
    return s


class Address(object):
    """Address object to contain everything known about an address"""
    def __init__(self,street=None,postcode=None,city=None,country=None):
        self.street = street
        self.postcode = postcode
        self.city = city
        self.country = country
        self.country_code = None
        self.phone = None
        self.name = None
        self.coord = None


    def geocode(self, language='en'):
        base_url = 'http://nominatim.openstreetmap.org/search?%s'
        params = { 'addressdetails': 1,
                   'limit': 1,
                   'format': 'xml',
                   'polygon': 0,
                   'accept-language': language}

        if self.country:
            t = etree.parse('/usr/share/xml/iso-codes/iso_3166.xml')
            r = t.xpath('//iso_3166_entry[@name="'+self.country+'"]')
            if len(r)==1:
                self.country_code = r[0].get("alpha_2_code")
        if self.country_code:
            params['countrycodes'] = self.country_code

        addrlist=[]
        addrlist.append(u''+c(self.name)+', '+c(self.street)+', '+c(self.city))
        addrlist.append(u''+c(self.street)+', '+c(self.postcode)+', '+c(self.city))
        addrlist.append(u''+c(self.street)+', '+c(self.city))

        for addr in addrlist:
            params['q'] = addr.encode('utf-8')
            url = base_url % urlencode(params)
            time.sleep(1)
            resp, content = h.request(url)
            root = etree.fromstring(content)
            places = int(root.xpath('count(//place[@place_id])'))
            if places == 1:
                place = root.find("place")
#                print etree.tostring(place,encoding='UTF-8',pretty_print=True)
                self.postcode = s(self.postcode,place.find("postcode"))
                self.city = s(self.city,place.find("city"))
                self.country = s(self.country,place.find("country"))
                self.country_code = s(self.country_code,place.find("country_code"))
                self.coord=Coord(place.get("lat"),place.get("lon"))
                return

    def add_phone(self, phone):
        self.phone = phone

    def set_name(self, name):
        self.name = name

    def db_xml(self):
        db = ElementMaker(namespace=const.DB_NS, nsmap=const.NSMAP)
        co = ''
        if self.coord:
            co = self.coord.db_xml()
        adr = db.address(db.street(c(self.street)),
                         db.postcode(c(self.postcode)),
                         db.city(c(self.city)),
                         db.country(c(self.country)),
                         db.phone(c(self.phone)),
                         co)
#                       type=self.type,
        return adr


def distance(p1, p2):
    res = Geodesic.WGS84.Inverse(p1[0], p1[1], p2[0], p2[1])
    return res['s12']

def mark(image, coord):
    draw = ImageDraw.Draw(image)
    x, y = coord
    r = 5
    bbox = (x-r, y-r, x+r, y+r)
    draw.ellipse(bbox, outline="red")

def box(image,box):
    draw = ImageDraw.Draw(image)
    draw.rectangle(box, outline="red")

def mapimages(coords, zoom=15,size=(TS,TS)):

        minlat = min(coords,key=attrgetter('latitude'))
        maxlat = max(coords,key=attrgetter('latitude'))
        minlon = min(coords,key=attrgetter('longitude'))
        maxlon = max(coords,key=attrgetter('longitude'))

        # Find minimal bounding box and expand it 5%
        hyp = distance((maxlat,minlon),(minlat,maxlon))
        hyp = hyp*0.05
        uld =  Geodesic.WGS84.Direct(maxlat, minlon, 315, hyp)
        urd =  Geodesic.WGS84.Direct(maxlat, maxlon,  45, hyp)
        lld =  Geodesic.WGS84.Direct(minlat, minlon, 225, hyp)
        lrd =  Geodesic.WGS84.Direct(minlat, maxlon, 135, hyp)

        ul = Coord(maxlat,minlon).direct(315, hyp)

        ul = (uld['lat2'],uld['lon2'])
        ur = (urd['lat2'],urd['lon2'])
        ll = (lld['lat2'],lld['lon2'])
        lr = (lrd['lat2'],lrd['lon2'])
        top = distance(ul,ur)
        bottom = distance(ll,lr)
        left = distance(ul,ll)
        right = distance(ur,lr)
#    m_per_pix = EC*abs(cos(lat))/2**(zoomlevel+8)
#    m_per_pix = ER*2*pi*abs(cos(lat))/2**(zoomlevel+8)
        for zoom in range(1,18):
            t = 2**(zoom)*TS/(ER*2*pi*abs(cos(ul[0])))
            toppix = t*top
            leftpix = t*left
            b = 2**(zoom)*TS/(ER*2*pi*abs(cos(ll[0])))
            bottompix = b*bottom
            rightpix  = b*right
#            print "Zoom: %s : %s %s %s %s" % (zoom, toppix, bottompix, leftpix, rightpix)
            if max(toppix,bottompix,leftpix,rightpix) > TS*4:
                break
#        print "Zoom to use : %s" % (zoom)
        ult = coordtile(ul,zoom)
        lrt = coordtile(lr,zoom)
        lattiles = range(int(ult[0]),int(lrt[0])+1)
        lontiles =  range(int(ult[1]),int(lrt[1])+1)
        size = (len(lontiles)*TS,len(lattiles)*TS)
        grid = Image.new("RGB", size, None)
        img = []
        for yi, y in enumerate(lattiles):
            for xi, x in enumerate(lontiles):
                url = tile(x,y,zoom)
#                print url
                time.sleep(1)
                request, content = h.request(url)
                img = Image.open(StringIO(content))
                box = (xi*TS, yi*TS)
                grid.paste(img, box)
        for c in coords:
            t = coordtile(c,zoom)
            o = offset(c,zoom)
            yp = [i for i,x in enumerate(lattiles) if x == int(t[0])][0]*TS+o[1]
            xp = [i for i,x in enumerate(lontiles) if x == int(t[1])][0]*TS+o[0]
            mark(grid, (xp,yp))

        t = coordtile(ul,zoom)
        o = offset(ul,zoom)
        yp = [i for i,x in enumerate(lattiles) if x == int(t[0])][0]*TS+o[1]
        xp = [i for i,x in enumerate(lontiles) if x == int(t[1])][0]*TS+o[0]
        t = coordtile(lr,zoom)
        o = offset(lr,zoom)
        ypl = [i for i,x in enumerate(lattiles) if x == int(t[0])][0]*TS+o[1]
        xpl = [i for i,x in enumerate(lontiles) if x == int(t[1])][0]*TS+o[0]
        gridc = grid.crop((xp,yp,xpl,ypl))
        gridc.show()
#        gridc.save("cap-un.png")

if __name__ == "__main__":
    for arg in sys.argv[1:]:
        al = arg.split("=")
        if al[0] == "lang":
            lang = al[1]
        if al[0] == "xptr":
            argument = al[1].decode('utf-8')

    addrlist = argument.split(',')
    addrfmt = "street,postcode,city,country"
    adict = addrfmt.split(',')
    argdict = dict(zip(adict,addrlist))
    addr = Address(**argdict)
    addr.geocode()
    axml = addr.db_xml()
#    clean_db(axml)

    #print(etree.tostring(cxml, pretty_print=True))
    #sys.stdout.write(out.encode('utf-8'))
    sys.stdout.write(etree.tostring(axml,encoding='UTF-8',pretty_print=False))
